import typing

from django.core.exceptions import ValidationError
from django.db import models
from django.utils import timezone
from django.utils.translation import gettext_lazy as _

from .crypto import KeyGenerator
from .utils import concatenate
from .validators import validate_timezone_aware


class BaseAPIKeyManager(models.Manager):
    key_generator = KeyGenerator()

    def assign_key(self, obj: "AbstractAPIKey") -> str:
        key, prefix, hashed_key = self.key_generator.generate()
        pk = concatenate(prefix, hashed_key)

        obj.id = pk
        obj.prefix = prefix
        obj.hashed_key = hashed_key

        return key

    def create_key(self, **kwargs: typing.Any) -> typing.Tuple["AbstractAPIKey", str]:
        # Prevent from manually setting the primary key.
        kwargs.pop("id", None)
        obj = self.model(**kwargs)
        obj.clean_fields()
        key = self.assign_key(obj)
        obj.save()
        return obj, key

    def get_usable_keys(self) -> models.QuerySet:
        return self.filter(revoked=False)

    def get_from_key(self, key: str) -> "AbstractAPIKey":
        prefix, _, _ = key.partition(".")
        queryset = self.get_usable_keys()

        try:
            api_key = queryset.get(prefix=prefix)
        except self.model.DoesNotExist:
            raise  # For the sake of being explicit.

        if not api_key.is_valid(key):
            raise self.model.DoesNotExist("Key is not valid.")
        else:
            return api_key

    def is_valid(self, key: str) -> bool:
        try:
            api_key = self.get_from_key(key)
        except self.model.DoesNotExist:
            return False

        if api_key.has_expired:
            return False

        return True


class APIKeyManager(BaseAPIKeyManager):
    pass


class AbstractAPIKey(models.Model):
    objects = APIKeyManager()

    id = models.CharField(max_length=150, unique=True, primary_key=True, editable=False)
    prefix = models.CharField(max_length=8, unique=True, editable=False)
    hashed_key = models.CharField(max_length=150, editable=False)
    created = models.DateTimeField(auto_now_add=True, db_index=True)
    name = models.CharField(
        max_length=50,
        blank=False,
        default=None,
        help_text=(_("A name for the API key. 50 characters max.")),
    )
    revoked = models.BooleanField(
        blank=True,
        default=False,
        help_text=(
            _(
                "If the API key is revoked, users cannot use it anymore. "
                "(This cannot be undone.)"
            )
        ),
    )
    expiry_date = models.DateTimeField(
        blank=True,
        null=True,
        verbose_name=_("Expires"),
        help_text=_("Once API key expires, users cannot use it anymore."),
        validators=[validate_timezone_aware],
    )

    class Meta:  # noqa
        abstract = True
        ordering = ("-created",)
        verbose_name = "API key"
        verbose_name_plural = "API keys"

    def __init__(self, *args: typing.Any, **kwargs: typing.Any):
        super().__init__(*args, **kwargs)
        # Store the initial value of `revoked` to detect changes.
        self._initial_revoked = self.revoked

    def _has_expired(self) -> bool:
        if self.expiry_date is None:
            return False
        return self.expiry_date < timezone.now()

    _has_expired.short_description = "Has expired"
    _has_expired.boolean = True
    has_expired = property(_has_expired)

    def is_valid(self, key: str) -> bool:
        key_generator = type(self).objects.key_generator
        return key_generator.verify(key, self.hashed_key)

    def clean(self) -> None:
        self._validate_revoked()

    def save(self, *args: typing.Any, **kwargs: typing.Any) -> None:
        self._validate_revoked()
        super().save(*args, **kwargs)

    def _validate_revoked(self) -> None:
        if self._initial_revoked and not self.revoked:
            raise ValidationError(
                _("The API key has been revoked, which cannot be undone.")
            )

    def __str__(self) -> str:
        return str(self.name)


class APIKey(AbstractAPIKey):
    pass



from django.conf import settings


def pytest_configure() -> None:
    settings.configure(
        **{
            "SECRET_KEY": "abcd",
            "USE_TZ": True,
            "INSTALLED_APPS": [
                # Mandatory
                "django.contrib.contenttypes",
                # Permissions
                "django.contrib.auth",
                # Admin
                "django.contrib.admin",
                "django.contrib.messages",
                "django.contrib.sessions",
                # Project
                "ninja_keys",
                "test_project.heroes",
            ],
            "TEMPLATES": [
                # Admin
                {
                    "BACKEND": "django.template.backends.django.DjangoTemplates",
                    "OPTIONS": {
                        "context_processors": [
                            "django.contrib.auth.context_processors.auth",
                            "django.contrib.messages.context_processors.messages",
                        ]
                    },
                }
            ],
            "MIDDLEWARE": [
                # Admin
                "django.contrib.sessions.middleware.SessionMiddleware",
                "django.contrib.messages.middleware.MessageMiddleware",
                "django.contrib.auth.middleware.AuthenticationMiddleware",
            ],
            "ROOT_URLCONF": "test_project.project.urls",
            "DATABASES": {
                "default": {
                    "ENGINE": "django.db.backends.sqlite3",
                    "NAME": ":memory:",
                },
                "test": {
                    "ENGINE": "django.db.backends.sqlite3",
                    "NAME": ":memory:",
                },
            },
        }
    )



from django.contrib import admin

from ninja_keys.admin import APIKeyModelAdmin

from .models import Hero, HeroAPIKey


@admin.register(HeroAPIKey)
class HeroAPIKeyModelAdmin(APIKeyModelAdmin):
    pass


@admin.register(Hero)
class HeroModelAdmin(admin.ModelAdmin):
    pass



# Generated by Django 5.1.6 on 2025-02-20 19:49

from django.db import migrations, models


class Migration(migrations.Migration):
    initial = True

    dependencies = []

    operations = [
        migrations.CreateModel(
            name="APIKey",
            fields=[
                (
                    "id",
                    models.CharField(
                        editable=False,
                        max_length=150,
                        primary_key=True,
                        serialize=False,
                        unique=True,
                    ),
                ),
                ("prefix", models.CharField(editable=False, max_length=8, unique=True)),
                ("hashed_key", models.CharField(editable=False, max_length=150)),
                ("created", models.DateTimeField(auto_now_add=True, db_index=True)),
                (
                    "name",
                    models.CharField(
                        default=None,
                        help_text="A name for the API key. 50 characters max.",
                        max_length=50,
                    ),
                ),
                (
                    "revoked",
                    models.BooleanField(
                        blank=True,
                        default=False,
                        help_text="If the API key is revoked, users cannot use it anymore. (This cannot be undone.)",
                    ),
                ),
                (
                    "expiry_date",
                    models.DateTimeField(
                        blank=True,
                        help_text="Once API key expires, users cannot use it anymore.",
                        null=True,
                        verbose_name="Expires",
                    ),
                ),
            ],
            options={
                "verbose_name": "API key",
                "verbose_name_plural": "API keys",
                "ordering": ("-created",),
                "abstract": False,
            },
        ),
    ]



import datetime

from django.conf import settings
from django.core.exceptions import ValidationError
from django.utils.timezone import is_naive
from django.utils.translation import gettext_lazy as _


def validate_timezone_aware(value: datetime.datetime) -> None:
    if settings.USE_TZ and is_naive(value):
        raise ValidationError(
            _("The expiry date must be timezone aware when USE_TZ is True.")
        )



import typing

from ninja.security import APIKeyHeader

from ninja_keys.models import AbstractAPIKey, APIKey


class BaseApiKeyAuth(APIKeyHeader):
    param_name = "X-API-Key"
    model: typing.Optional[typing.Type[AbstractAPIKey]] = None

    def authenticate(self, request, key):
        if not key:
            return None
        return self.model.objects.is_valid(key)


class ApiKeyAuth(BaseApiKeyAuth):
    model = APIKey



from ninja import NinjaAPI, Router

from test_project.heroes.auth import HeroAPIKeyAuth

api = NinjaAPI()
router = Router()


@router.get("/public")
def public(request):
    return "Hello, world!"


@router.get("/protected", auth=HeroAPIKeyAuth())
def protected(request):
    return "Hello, protected!"


api.add_router("", router)



import pytest
from django_test_migrations.migrator import Migrator

pytestmark = pytest.mark.skipif(
    Migrator is None,
    reason="django-test-migrations is not available",
)


@pytest.mark.django_db
def test_migrations_0001_initial(migrator: Migrator) -> None:
    old_state = migrator.apply_initial_migration(("ninja_keys", None))

    with pytest.raises(LookupError):
        old_state.apps.get_model("ninja_keys", "APIKey")

    new_state = migrator.apply_tested_migration(("ninja_keys", "0001_initial"))
    APIKey = new_state.apps.get_model("ninja_keys", "APIKey")
    assert APIKey.objects.count() == 0



import datetime
from typing import Callable

import pytest
from ninja import NinjaAPI
from ninja.responses import Response
from ninja.testing import TestClient

from ninja_keys.auth import ApiKeyAuth, BaseApiKeyAuth
from ninja_keys.models import APIKey
from tests.dateutils import NOW, TOMORROW

pytestmark = pytest.mark.django_db

api = NinjaAPI(version="2.0.0", auth=ApiKeyAuth())


@api.get("/test/")
def test_endpoint(request) -> Response:
    return Response(data={})


class ApiKeyWithoutXAuth(BaseApiKeyAuth):
    param_name = "Api-Key"
    model = APIKey


@api.get("/test_no_x/", auth=ApiKeyWithoutXAuth())
def test_new(request) -> Response:
    return Response(data={})


client = TestClient(api)


def test_if_valid_api_key_then_permission_granted() -> None:
    _, key = APIKey.objects.create_key(name="test")
    headers = {"X-API-Key": key}
    response = client.get("/test/", headers=headers)
    assert response.status_code == 200


def test_if_no_api_key_then_permission_denied() -> None:
    response = client.get("/test/")
    assert response.status_code == 401


def _scramble_prefix(key: str) -> str:
    prefix, _, secret_key = key.partition(".")
    truncated_prefix = prefix[:-1]
    return truncated_prefix + "." + secret_key


@pytest.mark.parametrize(
    "modifier",
    [
        lambda _: "",
        lambda _: "abcd",
        lambda _: "foo.bar",
        lambda key: " " + key,
        lambda key: key.upper(),
        lambda key: key.lower(),
        lambda key: _scramble_prefix(key),
    ],
)
def test_if_invalid_api_key_then_permission_denied(
    modifier: Callable[[str], str],
) -> None:
    _, key = APIKey.objects.create_key(name="test")
    headers = {"X-API-Key": modifier(key)}
    response = client.get("/test/", headers=headers)
    assert response.status_code == 401


def test_if_revoked_then_permission_denied() -> None:
    _, key = APIKey.objects.create_key(name="test", revoked=True)
    headers = {"X-API-Key": key}
    response = client.get("/test/", headers=headers)
    assert response.status_code == 401


TWO_DAYS_AGO = NOW - datetime.timedelta(days=2)


@pytest.mark.parametrize("expiry_date, ok", [(TOMORROW, True), (TWO_DAYS_AGO, False)])
def test_expiry_date(expiry_date: datetime.datetime, ok: bool) -> None:
    _, key = APIKey.objects.create_key(name="test", expiry_date=expiry_date)
    headers = {"X-API-Key": key}
    response = client.get("/test/", headers=headers)

    status_code = 200 if ok else 401
    assert response.status_code == status_code


def test_auth_param_name_override() -> None:
    _, key = APIKey.objects.create_key(name="test")
    headers = {"Api-Key": key}
    response = client.get("/test_no_x/", headers=headers)

    assert response.status_code == 200


def test_auth_param_name_override_with_x() -> None:
    _, key = APIKey.objects.create_key(name="test")
    headers = {"X-Api-Key": key}
    response = client.get("/test_no_x/", headers=headers)

    assert response.status_code == 401



import typing

from django.contrib import admin, messages
from django.db import models
from django.http.request import HttpRequest
from django.utils.translation import gettext_lazy as _

from .models import AbstractAPIKey, APIKey


class APIKeyModelAdmin(admin.ModelAdmin):
    model: typing.Type[AbstractAPIKey]

    list_display = (
        "prefix",
        "name",
        "created",
        "expiry_date",
        "_has_expired",
        "revoked",
    )
    list_filter = ("created",)
    search_fields = ("name", "prefix")

    def get_readonly_fields(
        self, request: HttpRequest, obj: models.Model = None
    ) -> typing.Tuple[str, ...]:
        obj = typing.cast(AbstractAPIKey, obj)
        fields: typing.Tuple[str, ...]

        fields = ("prefix",)
        if obj is not None and obj.revoked:
            fields = fields + ("name", "revoked", "expiry_date")

        return fields

    def save_model(
        self,
        request: HttpRequest,
        obj: AbstractAPIKey,
        form: typing.Any = None,
        change: bool = False,
    ) -> None:
        created = not obj.pk

        if created:
            key = self.model.objects.assign_key(obj)
            obj.save()
            message = _(
                "The API key for {} is: {}. ".format(obj.name, key)
                + "Please store it somewhere safe: "
                + "you will not be able to see it again."
            )
            messages.add_message(request, messages.WARNING, message)
        else:
            obj.save()


admin.site.register(APIKey, APIKeyModelAdmin)



import datetime as dt
import string

import pytest
from django.core.exceptions import ValidationError
from django.db.utils import IntegrityError
from django.test import override_settings
from test_project.heroes.models import Hero, HeroAPIKey

from ninja_keys.models import APIKey

from .dateutils import NOW, TOMORROW, YESTERDAY

pytestmark = pytest.mark.django_db


def test_key_generation() -> None:
    api_key, generated_key = APIKey.objects.create_key(name="test")
    prefix = api_key.prefix
    hashed_key = api_key.hashed_key

    assert prefix and hashed_key

    charset = set(string.ascii_letters + string.digits + ".")
    assert all(c in charset for c in generated_key)

    # The generated key must be valid…
    assert api_key.is_valid(generated_key) is True

    # But not the hashed key.
    assert api_key.is_valid(hashed_key) is False


def test_name_is_required() -> None:
    with pytest.raises(IntegrityError):
        APIKey.objects.create()


def test_cannot_unrevoke() -> None:
    api_key, _ = APIKey.objects.create_key(name="test", revoked=True)

    # Try to unrevoke the API key programmatically.
    api_key.revoked = False

    with pytest.raises(ValidationError):
        api_key.save()

    with pytest.raises(ValidationError):
        api_key.clean()


@pytest.mark.parametrize(
    "expiry_date, has_expired",
    [(None, False), (NOW, True), (TOMORROW, False), (YESTERDAY, True)],
)
def test_has_expired(expiry_date: dt.datetime, has_expired: bool) -> None:
    api_key, _ = APIKey.objects.create_key(name="test", expiry_date=expiry_date)
    assert api_key.has_expired is has_expired


def test_timezone_naive_with_use_tz_true() -> None:
    with pytest.raises(ValidationError):
        APIKey.objects.create_key(name="test", expiry_date=dt.datetime.now(tz=None))


@override_settings(USE_TZ=False)
def test_timezone_aware_with_use_tz_false() -> None:
    with pytest.raises(ValueError):
        APIKey.objects.create_key(
            name="test", expiry_date=dt.datetime.now(tz=dt.timezone.utc)
        )


def test_custom_api_key_model() -> None:
    hero = Hero.objects.create()
    hero_api_key, generated_key = HeroAPIKey.objects.create_key(name="test", hero=hero)
    assert hero_api_key.is_valid(generated_key)
    assert hero_api_key.hero.id == hero.id
    assert hero.api_keys.first() == hero_api_key


@pytest.mark.django_db
def test_api_key_manager_get_from_key() -> None:
    api_key, generated_key = APIKey.objects.create_key(name="test")
    retrieved_key = APIKey.objects.get_from_key(generated_key)
    assert retrieved_key == api_key


@pytest.mark.django_db
def test_api_key_manager_get_from_key_missing_key() -> None:
    with pytest.raises(APIKey.DoesNotExist):
        APIKey.objects.get_from_key("foobar")


@pytest.mark.django_db
def test_api_key_manager_get_from_key_invalid_key() -> None:
    api_key, generated_key = APIKey.objects.create_key(name="test")
    prefix, _, _ = generated_key.partition(".")
    invalid_key = f"{prefix}.foobar"
    with pytest.raises(APIKey.DoesNotExist):
        APIKey.objects.get_from_key(invalid_key)


def test_api_key_str() -> None:
    _, generated_key = APIKey.objects.create_key(name="test")
    retrieved_key = APIKey.objects.get_from_key(generated_key)
    assert str(retrieved_key) == "test"


@pytest.mark.django_db
def test_api_key_manager_is_valid_missing_key() -> None:
    assert APIKey.objects.is_valid("foobar") is False


@pytest.mark.django_db
def test_api_key_manager_is_valid_expired_key() -> None:
    _, key = APIKey.objects.create_key(name="test", expiry_date=YESTERDAY)
    assert APIKey.objects.is_valid(key) is False



import pytest
from django.contrib.admin import site
from django.contrib.messages import get_messages
from django.contrib.messages.middleware import MessageMiddleware
from django.contrib.sessions.middleware import SessionMiddleware
from django.http.request import HttpRequest
from django.http.response import HttpResponse
from django.test import RequestFactory
from test_project.heroes.admin import HeroAPIKeyModelAdmin
from test_project.heroes.models import Hero, HeroAPIKey

from ninja_keys.admin import APIKeyModelAdmin
from ninja_keys.models import APIKey


def build_admin_request(rf: RequestFactory) -> HttpRequest:
    request = rf.post("/")

    def get_response(request: HttpRequest) -> HttpResponse:
        raise NotImplementedError  # pragma: no cover  # Unused in these tests.

    # NOTE: all middleware must be instantiated before
    # any middleware can process the request.
    sessions = SessionMiddleware(get_response)
    messages = MessageMiddleware(sessions.get_response)

    sessions.process_request(request)
    messages.process_request(request)

    return request


@pytest.mark.django_db
def test_admin_readonly_fields(rf: RequestFactory) -> None:
    request = build_admin_request(rf)

    admin = APIKeyModelAdmin(APIKey, site)

    assert admin.get_readonly_fields(request) == ("prefix",)

    api_key = APIKey(name="test")
    assert admin.get_readonly_fields(request, obj=api_key) == ("prefix",)

    api_key = APIKey(name="test", revoked=True)
    assert admin.get_readonly_fields(request, obj=api_key) == (
        "prefix",
        "name",
        "revoked",
        "expiry_date",
    )


@pytest.mark.django_db
def test_admin_create_api_key(rf: RequestFactory) -> None:
    request = build_admin_request(rf)

    admin = APIKeyModelAdmin(APIKey, site)
    api_key = APIKey(name="test")

    assert not api_key.pk
    admin.save_model(request, obj=api_key)
    assert api_key.pk

    messages = get_messages(request)
    assert len(messages) == 1


@pytest.mark.django_db
def test_admin_create_custom_api_key(rf: RequestFactory) -> None:
    request = build_admin_request(rf)

    admin = HeroAPIKeyModelAdmin(HeroAPIKey, site)
    api_key = HeroAPIKey(name="test", hero=Hero.objects.create())

    assert not api_key.pk
    admin.save_model(request, obj=api_key)
    assert api_key.pk

    messages = get_messages(request)
    assert len(messages) == 1


@pytest.mark.django_db
def test_admin_update_api_key(rf: RequestFactory) -> None:
    request = build_admin_request(rf)

    admin = APIKeyModelAdmin(APIKey, site)
    api_key, _ = APIKey.objects.create_key(name="test")

    api_key.name = "another-test"
    admin.save_model(request, obj=api_key)
    refreshed = APIKey.objects.get(pk=api_key.pk)
    assert refreshed.name == "another-test"



from django.db import models

from ninja_keys.models import AbstractAPIKey, BaseAPIKeyManager


class Hero(models.Model):
    objects = models.Manager()
    name = models.CharField(max_length=64)
    retired = models.BooleanField(default=False)

    class Meta:
        verbose_name_plural = "Heroes"

    def __str__(self) -> str:
        return self.name


class HeroAPIKeyManager(BaseAPIKeyManager):
    def get_usable_keys(self) -> models.QuerySet:
        return super().get_usable_keys().filter(hero__retired=False)


class HeroAPIKey(AbstractAPIKey):
    objects = HeroAPIKeyManager()
    hero = models.ForeignKey(Hero, on_delete=models.CASCADE, related_name="api_keys")

    class Meta(AbstractAPIKey.Meta):
        verbose_name = "Hero API key"
        verbose_name_plural = "Hero API keys"



import pytest
from django.test import Client
from test_project.heroes.models import Hero, HeroAPIKey


@pytest.mark.django_db
def test_test_project_routes() -> None:
    batman = Hero.objects.create(name="Batman")
    _, key = HeroAPIKey.objects.create_key(name="test", hero=batman)
    headers = {"HTTP_X-API-Key": key}

    client = Client()

    response = client.get("/api/public", format="json")
    assert response.status_code == 200

    response = client.get("/api/protected", format="json", **headers)
    assert response.status_code == 200

    # response = client.get("/api/protected/object/", format="json", **headers)
    # assert response.status_code == 200


@pytest.mark.django_db
def test_test_project_routes_with_empty_key() -> None:
    headers = {"HTTP_X-API-Key": ""}

    client = Client()

    response = client.get("/api/public", format="json", **headers)
    assert response.status_code == 200

    response = client.get("/api/protected", format="json", **headers)
    assert response.status_code == 401



import hashlib
import typing

from django.contrib.auth.hashers import (
    BasePasswordHasher,
    make_password,
)
from django.utils.crypto import constant_time_compare, get_random_string

from ninja_keys.utils import concatenate


class Sha512ApiKeyHasher(BasePasswordHasher):
    """
    An API key hasher using the sha512 algorithm.

    This hasher should *NEVER* be used in Django's `PASSWORD_HASHERS` setting.
    It is insecure for use in hashing passwords, but is safe for hashing
    high entropy, randomly generated API keys.
    """

    algorithm = "sha512"

    def salt(self) -> str:
        """No need for a salt on a high entropy key."""
        return ""

    def encode(self, password: str, salt: str) -> str:
        if salt != "":
            raise ValueError("salt is unnecessary for high entropy API tokens.")
        hash_value = hashlib.sha512(password.encode()).hexdigest()
        return f"{str(self.algorithm)}$${str(hash_value)}"

    def verify(self, password: str, encoded: str) -> bool:
        encoded_2 = self.encode(password, "")
        return constant_time_compare(encoded, encoded_2)


class KeyGenerator:
    def __init__(self, prefix_length: int = 8, secret_key_length: int = 32, hasher: BasePasswordHasher = None):
        self.prefix_length = prefix_length
        self.secret_key_length = secret_key_length
        self.hasher = hasher or Sha512ApiKeyHasher()

    def get_prefix(self) -> str:
        return get_random_string(self.prefix_length)

    def get_secret_key(self) -> str:
        return get_random_string(self.secret_key_length)

    def hash(self, value: str) -> str:
        return make_password(value, hasher=self.hasher)

    def generate(self) -> typing.Tuple[str, str, str]:
        prefix = self.get_prefix()
        secret_key = self.get_secret_key()
        key = concatenate(prefix, secret_key)
        hashed_key = self.hash(key)
        return key, prefix, hashed_key

    def verify(self, key: str, hashed_key: str) -> bool:
        return self.hasher.verify(key, hashed_key)




import os

BASE_DIR = os.path.dirname(os.path.dirname(os.path.abspath(__file__)))
SECRET_KEY = "django-insecure-r_nyoza3g9qgqrboje+$(c57be4pxqnp1y-qg2==iu)tyr_n^h"
DEBUG = True

ALLOWED_HOSTS = ["localhost", "127.0.0.1", "::1"]

# Application definition

INSTALLED_APPS = [
    "django.contrib.admin",
    "django.contrib.auth",
    "django.contrib.contenttypes",
    "django.contrib.sessions",
    "django.contrib.messages",
    "django.contrib.staticfiles",
    "ninja_keys",
    "test_project.heroes",
]

MIDDLEWARE = [
    "django.middleware.security.SecurityMiddleware",
    "django.contrib.sessions.middleware.SessionMiddleware",
    "django.middleware.common.CommonMiddleware",
    "django.middleware.csrf.CsrfViewMiddleware",
    "django.contrib.auth.middleware.AuthenticationMiddleware",
    "django.contrib.messages.middleware.MessageMiddleware",
    "django.middleware.clickjacking.XFrameOptionsMiddleware",
]

ROOT_URLCONF = "project.urls"

TEMPLATES = [
    {
        "BACKEND": "django.template.backends.django.DjangoTemplates",
        "DIRS": [],
        "APP_DIRS": True,
        "OPTIONS": {
            "context_processors": [
                "django.template.context_processors.debug",
                "django.template.context_processors.request",
                "django.contrib.auth.context_processors.auth",
                "django.contrib.messages.context_processors.messages",
            ],
        },
    },
]

WSGI_APPLICATION = "project.wsgi.application"


# Database
# https://docs.djangoproject.com/en/5.1/ref/settings/#databases

DATABASES = {
    "default": {
        "ENGINE": "django.db.backends.sqlite3",
        "NAME": os.path.join(BASE_DIR, "db.sqlite3"),
    }
}


# Password validation
# https://docs.djangoproject.com/en/5.1/ref/settings/#auth-password-validators

AUTH_PASSWORD_VALIDATORS = [
    {
        "NAME": "django.contrib.auth.password_validation.UserAttributeSimilarityValidator",
    },
    {
        "NAME": "django.contrib.auth.password_validation.MinimumLengthValidator",
    },
    {
        "NAME": "django.contrib.auth.password_validation.CommonPasswordValidator",
    },
    {
        "NAME": "django.contrib.auth.password_validation.NumericPasswordValidator",
    },
]


# Internationalization
# https://docs.djangoproject.com/en/5.1/topics/i18n/

LANGUAGE_CODE = "en-us"

TIME_ZONE = "UTC"

USE_I18N = True

USE_TZ = True


# Static files (CSS, JavaScript, Images)
# https://docs.djangoproject.com/en/5.1/howto/static-files/

STATIC_URL = "static/"

# Default primary key field type
# https://docs.djangoproject.com/en/5.1/ref/settings/#default-auto-field

DEFAULT_AUTO_FIELD = "django.db.models.BigAutoField"



import pytest

from ninja_keys.crypto import Sha512ApiKeyHasher


def test_sha512hasher_encode() -> None:
    hasher = Sha512ApiKeyHasher()

    key = "test"
    hashed_key = hasher.encode(key, "")
    assert hasher.verify(key, hashed_key)
    assert not hasher.verify("not-test", hashed_key)


def test_sha512hasher_invalid_salt() -> None:
    hasher = Sha512ApiKeyHasher()
    with pytest.raises(ValueError):
        hasher.encode("test", "salt")